@@ -7,6 +7,7 @@ use alloc::vec::Vec;
 use core::{char, cmp::Ordering, ops::RangeBounds};
 
 use crate::{uniset::UnicodeSet, utils::deconstruct_range};
+use zerovec::{ule::AsULE, ZeroVec};
 
 /// A builder for [`UnicodeSet`].
 ///
@@ -23,8 +24,10 @@ impl UnicodeSetBuilder {
     }
 
     /// Returns a [`UnicodeSet`] and consumes the [`UnicodeSetBuilder`]
-    pub fn build(self) -> UnicodeSet {
-        UnicodeSet::from_inversion_list(self.intervals).unwrap()
+    pub fn build(self) -> UnicodeSet<'static> {
+        let inv_list: ZeroVec<u32> = ZeroVec::alloc_from_slice(&self.intervals);
+        #[allow(clippy::unwrap_used)] // TODO(#1668) Clippy exceptions need docs or fixing.
+        UnicodeSet::from_inversion_list(inv_list).unwrap()
     }
 
     /// Abstraction for adding/removing a range from start..end
@@ -41,13 +44,18 @@ impl UnicodeSetBuilder {
 
         if start_eq_end && start_pos_check && end_res.is_err() {
             let ins = &[start, end];
-            self.intervals.splice(start_ind..end_ind, ins.iter().copied());
+            self.intervals
+                .splice(start_ind..end_ind, ins.iter().copied());
         } else {
+            #[allow(clippy::indexing_slicing)]
             if start_pos_check {
+                // TODO(#1668) Clippy exceptions need docs or fixing.
                 self.intervals[start_ind] = start;
                 start_ind += 1;
             }
             if end_pos_check {
+                #[allow(clippy::indexing_slicing)]
+                // TODO(#1668) Clippy exceptions need docs or fixing.
                 if end_res.is_ok() {
                     end_ind += 1;
                 } else {
@@ -82,7 +90,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_char('a');
     /// let check = builder.build();
@@ -103,7 +111,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_u32(0x41);
     /// let check = builder.build();
@@ -121,7 +129,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='Z'));
     /// let check = builder.build();
@@ -132,22 +140,51 @@ impl UnicodeSetBuilder {
         self.add(start, end);
     }
 
+    /// Add the range of characters, represented as u32, to the [`UnicodeSetBuilder`]
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu_uniset::UnicodeSetBuilder;
+    /// let mut builder = UnicodeSetBuilder::new();
+    /// builder.add_range_u32(&(0xd800..=0xdfff));
+    /// let check = builder.build();
+    /// assert_eq!(check.contains_u32(0xd900), true);
+    /// ```
+    pub fn add_range_u32(&mut self, range: &impl RangeBounds<u32>) {
+        let (start, end) = deconstruct_range(range);
+        if start <= end && end <= char::MAX as u32 {
+            self.add(start, end);
+        }
+    }
+
     /// Add the [`UnicodeSet`] reference to the [`UnicodeSetBuilder`]
     ///
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSet, UnicodeSetBuilder};
+    /// use icu_uniset::{UnicodeSet, UnicodeSetBuilder};
     /// let mut builder = UnicodeSetBuilder::new();
-    /// let set = UnicodeSet::from_inversion_list(vec![0x41, 0x4C]).unwrap();
+    /// let set = UnicodeSet::from_inversion_list_slice(&[0x41, 0x4C]).unwrap();
     /// builder.add_set(&set);
     /// let check = builder.build();
     /// assert_eq!(check.iter_chars().next(), Some('A'));
     /// ```
+    #[allow(unused_assignments)]
     pub fn add_set(&mut self, set: &UnicodeSet) {
-        for range in set.as_inversion_list().chunks(2) {
-            self.add(range[0], range[1]);
-        }
+        set.as_inversion_list()
+            .as_ule_slice()
+            .chunks(2)
+            .for_each(|pair| {
+                self.add(
+                    #[allow(clippy::indexing_slicing)]
+                    // TODO(#1668) Clippy exceptions need docs or fixing.
+                    AsULE::from_unaligned(pair[0]),
+                    #[allow(clippy::indexing_slicing)]
+                    // TODO(#1668) Clippy exceptions need docs or fixing.
+                    AsULE::from_unaligned(pair[1]),
+                )
+            });
     }
 
     /// Removes the range from the [`UnicodeSetBuilder`]
@@ -159,6 +196,7 @@ impl UnicodeSetBuilder {
             return;
         }
         if let Some(last) = self.intervals.last() {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             if start <= self.intervals[0] && end >= *last {
                 self.intervals.clear();
             } else {
@@ -172,7 +210,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='Z'));
     /// builder.remove_char('A');
@@ -188,7 +226,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='Z'));
     /// builder.remove_range(&('A'..='C'));
@@ -204,17 +242,28 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSet, UnicodeSetBuilder};
+    /// use icu_uniset::{UnicodeSet, UnicodeSetBuilder};
     /// let mut builder = UnicodeSetBuilder::new();
-    /// let set = UnicodeSet::from_inversion_list(vec![0x41, 0x46]).unwrap();
+    /// let set = UnicodeSet::from_inversion_list_slice(&[0x41, 0x46]).unwrap();
     /// builder.add_range(&('A'..='Z'));
     /// builder.remove_set(&set); // removes 'A'..='E'
     /// let check = builder.build();
     /// assert_eq!(check.iter_chars().next(), Some('F'));
+    #[allow(unused_assignments)]
     pub fn remove_set(&mut self, set: &UnicodeSet) {
-        for range in set.as_inversion_list().chunks(2) {
-            self.remove(range[0], range[1]);
-        }
+        set.as_inversion_list()
+            .as_ule_slice()
+            .chunks(2)
+            .for_each(|pair| {
+                self.remove(
+                    #[allow(clippy::indexing_slicing)]
+                    // TODO(#1668) Clippy exceptions need docs or fixing.
+                    AsULE::from_unaligned(pair[0]),
+                    #[allow(clippy::indexing_slicing)]
+                    // TODO(#1668) Clippy exceptions need docs or fixing.
+                    AsULE::from_unaligned(pair[1]),
+                )
+            });
     }
 
     /// Retain the specified character in the [`UnicodeSetBuilder`] if it exists
@@ -222,7 +271,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='Z'));
     /// builder.retain_char('A');
@@ -242,7 +291,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='Z'));
     /// builder.retain_range(&('A'..='B'));
@@ -263,20 +312,25 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSetBuilder, UnicodeSet};
+    /// use icu_uniset::{UnicodeSetBuilder, UnicodeSet};
     /// let mut builder = UnicodeSetBuilder::new();
-    /// let set = UnicodeSet::from_inversion_list(vec![65, 70]).unwrap();
+    /// let set = UnicodeSet::from_inversion_list_slice(&[65, 70]).unwrap();
     /// builder.add_range(&('A'..='Z'));
     /// builder.retain_set(&set); // retains 'A'..='E'
     /// let check = builder.build();
     /// assert!(check.contains('A'));
     /// assert!(!check.contains('G'));
     /// ```
+    #[allow(unused_assignments)]
     pub fn retain_set(&mut self, set: &UnicodeSet) {
         let mut prev = 0;
-        for range in set.as_inversion_list().chunks(2) {
-            self.remove(prev, range[0]);
-            prev = range[1];
+        for pair in set.as_inversion_list().as_ule_slice().chunks(2) {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+            let range_start = AsULE::from_unaligned(pair[0]);
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+            let range_limit = AsULE::from_unaligned(pair[1]);
+            self.remove(prev, range_start);
+            prev = range_limit;
         }
         self.remove(prev, (char::MAX as u32) + 1);
     }
@@ -286,20 +340,20 @@ impl UnicodeSetBuilder {
     ///
     /// Performs in `O(B + S)`, where `B` is the number of endpoints in the Builder, and `S` is the number
     /// of endpoints in the argument.
-    fn complement_list(&mut self, set: &[u32]) {
+    fn complement_list(&mut self, set_iter: impl core::iter::Iterator<Item = u32>) {
         let mut res: Vec<u32> = vec![]; // not the biggest fan of having to allocate new memory
         let mut ai = self.intervals.iter();
-        let mut bi = set.iter();
+        let mut bi = set_iter;
         let mut a = ai.next();
         let mut b = bi.next();
         while let (Some(c), Some(d)) = (a, b) {
-            match c.cmp(d) {
+            match c.cmp(&d) {
                 Ordering::Less => {
                     res.push(*c);
                     a = ai.next();
                 }
                 Ordering::Greater => {
-                    res.push(*d);
+                    res.push(d);
                     b = bi.next();
                 }
                 Ordering::Equal => {
@@ -312,7 +366,7 @@ impl UnicodeSetBuilder {
             res.push(*c)
         }
         if let Some(d) = b {
-            res.push(*d)
+            res.push(d)
         }
         res.extend(ai);
         res.extend(bi);
@@ -325,9 +379,9 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSetBuilder, UnicodeSet};
+    /// use icu_uniset::{UnicodeSetBuilder, UnicodeSet};
     /// let mut builder = UnicodeSetBuilder::new();
-    /// let set = UnicodeSet::from_inversion_list(vec![0x0, 0x41, 0x46, (std::char::MAX as u32) + 1]).unwrap();
+    /// let set = UnicodeSet::from_inversion_list_slice(&[0x0, 0x41, 0x46, (std::char::MAX as u32) + 1]).unwrap();
     /// builder.add_set(&set);
     /// builder.complement();
     /// let check = builder.build();
@@ -335,6 +389,7 @@ impl UnicodeSetBuilder {
     /// ```
     pub fn complement(&mut self) {
         if !self.intervals.is_empty() {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             if self.intervals[0] == 0 {
                 self.intervals.drain(0..1);
             } else {
@@ -345,6 +400,9 @@ impl UnicodeSetBuilder {
             } else {
                 self.intervals.push((char::MAX as u32) + 1);
             }
+        } else {
+            self.intervals
+                .extend_from_slice(&[0, (char::MAX as u32 + 1)]);
         }
     }
 
@@ -353,7 +411,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='D'));
     /// builder.complement_char('A');
@@ -365,7 +423,7 @@ impl UnicodeSetBuilder {
     pub fn complement_char(&mut self, c: char) {
         let code_point = c as u32;
         let to_complement = [code_point, code_point + 1];
-        self.complement_list(&to_complement);
+        self.complement_list(to_complement.iter().copied());
     }
 
     /// Complements the range in the builder, adding any elements in the range if not in the builder, and
@@ -374,7 +432,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::UnicodeSetBuilder;
+    /// use icu_uniset::UnicodeSetBuilder;
     /// let mut builder = UnicodeSetBuilder::new();
     /// builder.add_range(&('A'..='D'));
     /// builder.complement_range(&('C'..='F'));
@@ -385,7 +443,7 @@ impl UnicodeSetBuilder {
     pub fn complement_range(&mut self, range: &impl RangeBounds<char>) {
         let (start, end) = deconstruct_range(range);
         let to_complement = [start, end];
-        self.complement_list(&to_complement);
+        self.complement_list(to_complement.iter().copied());
     }
 
     /// Complements the set in the builder, adding any elements in the set if not in the builder, and
@@ -394,9 +452,9 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSetBuilder, UnicodeSet};
+    /// use icu_uniset::{UnicodeSetBuilder, UnicodeSet};
     /// let mut builder = UnicodeSetBuilder::new();
-    /// let set = UnicodeSet::from_inversion_list(vec![0x41, 0x46, 0x4B, 0x5A]).unwrap();
+    /// let set = UnicodeSet::from_inversion_list_slice(&[0x41, 0x46, 0x4B, 0x5A]).unwrap();
     /// builder.add_range(&('C'..='N')); // 67 - 78
     /// builder.complement_set(&set);
     /// let check = builder.build();
@@ -404,7 +462,8 @@ impl UnicodeSetBuilder {
     /// assert!(!check.contains('N')); // 78
     /// ```
     pub fn complement_set(&mut self, set: &UnicodeSet) {
-        self.complement_list(set.as_inversion_list());
+        let inv_list_iter_owned = set.as_inversion_list().iter();
+        self.complement_list(inv_list_iter_owned);
     }
 
     /// Returns whether the build is empty.
@@ -412,7 +471,7 @@ impl UnicodeSetBuilder {
     /// # Examples
     ///
     /// ```
-    /// use icu::uniset::{UnicodeSetBuilder, UnicodeSet};
+    /// use icu_uniset::{UnicodeSetBuilder, UnicodeSet};
     /// let mut builder = UnicodeSetBuilder::new();
     /// let check = builder.build();
     /// assert!(check.is_empty());
@@ -426,9 +485,11 @@ impl UnicodeSetBuilder {
 mod tests {
     use super::{UnicodeSet, UnicodeSetBuilder};
     use core::char;
+    use zerovec::ZeroVec;
 
     fn generate_tester(ex: Vec<u32>) -> UnicodeSetBuilder {
-        let check = UnicodeSet::from_inversion_list(ex).unwrap();
+        let inv_list: ZeroVec<u32> = ZeroVec::alloc_from_slice(&ex);
+        let check = UnicodeSet::from_inversion_list(inv_list).unwrap();
         let mut builder = UnicodeSetBuilder::new();
         builder.add_set(&check);
         builder
@@ -611,7 +672,8 @@ mod tests {
     #[test]
     fn test_add_unicodeset() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32]);
-        let check = UnicodeSet::from_inversion_list(vec![0x5, 0xA, 0x16, 0x21, 0x2C, 0x33]).unwrap();
+        let check =
+            UnicodeSet::from_inversion_list_slice(&[0x5, 0xA, 0x16, 0x21, 0x2C, 0x33]).unwrap();
         builder.add_set(&check);
         let expected = vec![0x5, 0x14, 0x16, 0x21, 0x28, 0x33];
         assert_eq!(builder.intervals, expected);
@@ -632,6 +694,14 @@ mod tests {
         assert_eq!(builder.intervals, expected);
     }
 
+    #[test]
+    fn test_add_range_u32() {
+        let mut builder = UnicodeSetBuilder::new();
+        builder.add_range_u32(&(0xd800..=0xdfff));
+        let expected = vec![0xd800, 0xe000];
+        assert_eq!(builder.intervals, expected);
+    }
+
     #[test]
     fn test_add_invalid_range() {
         let mut builder = UnicodeSetBuilder::new();
@@ -737,7 +807,7 @@ mod tests {
     #[test]
     fn test_remove_set() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32, 70, 80]);
-        let remove = UnicodeSet::from_inversion_list(vec![0xA, 0x14, 0x2D, 0x4B]).unwrap();
+        let remove = UnicodeSet::from_inversion_list_slice(&[0xA, 0x14, 0x2D, 0x4B]).unwrap();
         builder.remove_set(&remove);
         let expected = vec![0x28, 0x2D, 0x4B, 0x50];
         assert_eq!(builder.intervals, expected);
@@ -769,7 +839,8 @@ mod tests {
     #[test]
     fn test_retain_set() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32, 70, 80]);
-        let retain = UnicodeSet::from_inversion_list(vec![0xE, 0x14, 0x19, 0x37, 0x4D, 0x51]).unwrap();
+        let retain =
+            UnicodeSet::from_inversion_list_slice(&[0xE, 0x14, 0x19, 0x37, 0x4D, 0x51]).unwrap();
         builder.retain_set(&retain);
         let expected = vec![0xE, 0x14, 0x28, 0x32, 0x4D, 0x50];
         assert_eq!(builder.intervals, expected);
@@ -783,6 +854,18 @@ mod tests {
         assert_eq!(builder.intervals, expected);
     }
 
+    #[test]
+    fn test_complement_empty() {
+        let mut builder = generate_tester(vec![]);
+        builder.complement();
+        let expected = vec![0x0, (char::MAX as u32) + 1];
+        assert_eq!(builder.intervals, expected);
+
+        builder.complement();
+        let expected: Vec<u32> = vec![];
+        assert_eq!(builder.intervals, expected);
+    }
+
     #[test]
     fn test_complement_zero_max() {
         let mut builder = generate_tester(vec![0x0, 0xA, 0x5A, (char::MAX as u32) + 1]);
@@ -794,7 +877,7 @@ mod tests {
     #[test]
     fn test_complement_interior() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32]);
-        builder.complement_list(&[0xE, 0x14]);
+        builder.complement_list([0xE, 0x14].iter().copied());
         let expected = vec![0xA, 0xE, 0x28, 0x32];
         assert_eq!(builder.intervals, expected);
     }
@@ -802,7 +885,7 @@ mod tests {
     #[test]
     fn test_complement_exterior() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32]);
-        builder.complement_list(&[0x19, 0x23]);
+        builder.complement_list([0x19, 0x23].iter().copied());
         let expected = vec![0xA, 0x14, 0x19, 0x23, 0x28, 0x32];
         assert_eq!(builder.intervals, expected);
     }
@@ -810,7 +893,7 @@ mod tests {
     #[test]
     fn test_complement_larger_list() {
         let mut builder = generate_tester(vec![0xA, 0x14, 0x28, 0x32]);
-        builder.complement_list(&[0x1E, 0x37, 0x3C, 0x46]);
+        builder.complement_list([0x1E, 0x37, 0x3C, 0x46].iter().copied());
         let expected = vec![0xA, 0x14, 0x1E, 0x28, 0x32, 0x37, 0x3C, 0x46];
         assert_eq!(builder.intervals, expected);
     }
@@ -835,7 +918,7 @@ mod tests {
     #[test]
     fn test_complement_set() {
         let mut builder = generate_tester(vec![0x43, 0x4E]);
-        let set = UnicodeSet::from_inversion_list(vec![0x41, 0x46, 0x4B, 0x5A]).unwrap();
+        let set = UnicodeSet::from_inversion_list_slice(&[0x41, 0x46, 0x4B, 0x5A]).unwrap();
         builder.complement_set(&set);
         let expected = vec![0x41, 0x43, 0x46, 0x4B, 0x4E, 0x5A];
         assert_eq!(builder.intervals, expected);
@@ -844,6 +927,6 @@ mod tests {
     #[test]
     fn test_is_empty() {
         let mut builder = UnicodeSetBuilder::new();
-        assert_eq!(true, builder.is_empty());
+        assert!(builder.is_empty());
     }
 }
