@@ -5,24 +5,30 @@
 use alloc::vec;
 use core::{
     convert::TryFrom,
+    iter::FromIterator,
     ops::{Range, RangeBounds, RangeFrom, RangeFull, RangeInclusive, RangeTo, RangeToInclusive},
 };
 
 use super::UnicodeSetError;
 use crate::utils::deconstruct_range;
-use crate::UnicodeSet;
+use crate::{UnicodeSet, UnicodeSetBuilder};
+use zerovec::ZeroVec;
 
-fn try_from_range(range: &impl RangeBounds<char>) -> Result<UnicodeSet, UnicodeSetError> {
+fn try_from_range<'data, 'r>(
+    range: &'r impl RangeBounds<char>,
+) -> Result<UnicodeSet<'data>, UnicodeSetError> {
     let (from, till) = deconstruct_range(range);
     if from < till {
         let set = vec![from, till];
-        Ok(UnicodeSet::from_inversion_list(set).unwrap())
+        let inv_list: ZeroVec<u32> = ZeroVec::alloc_from_slice(&set);
+        #[allow(clippy::unwrap_used)] // TODO(#1668) Clippy exceptions need docs or fixing.
+        Ok(UnicodeSet::from_inversion_list(inv_list).unwrap())
     } else {
         Err(UnicodeSetError::InvalidRange(from, till))
     }
 }
 
-impl TryFrom<&Range<char>> for UnicodeSet {
+impl<'data> TryFrom<&Range<char>> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(range: &Range<char>) -> Result<Self, Self::Error> {
@@ -30,7 +36,7 @@ impl TryFrom<&Range<char>> for UnicodeSet {
     }
 }
 
-impl TryFrom<&RangeFrom<char>> for UnicodeSet {
+impl<'data> TryFrom<&RangeFrom<char>> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(range: &RangeFrom<char>) -> Result<Self, Self::Error> {
@@ -38,7 +44,7 @@ impl TryFrom<&RangeFrom<char>> for UnicodeSet {
     }
 }
 
-impl TryFrom<&RangeFull> for UnicodeSet {
+impl<'data> TryFrom<&RangeFull> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(_: &RangeFull) -> Result<Self, Self::Error> {
@@ -46,7 +52,7 @@ impl TryFrom<&RangeFull> for UnicodeSet {
     }
 }
 
-impl TryFrom<&RangeInclusive<char>> for UnicodeSet {
+impl<'data> TryFrom<&RangeInclusive<char>> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(range: &RangeInclusive<char>) -> Result<Self, Self::Error> {
@@ -54,7 +60,7 @@ impl TryFrom<&RangeInclusive<char>> for UnicodeSet {
     }
 }
 
-impl TryFrom<&RangeTo<char>> for UnicodeSet {
+impl<'data> TryFrom<&RangeTo<char>> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(range: &RangeTo<char>) -> Result<Self, Self::Error> {
@@ -62,7 +68,7 @@ impl TryFrom<&RangeTo<char>> for UnicodeSet {
     }
 }
 
-impl TryFrom<&RangeToInclusive<char>> for UnicodeSet {
+impl<'data> TryFrom<&RangeToInclusive<char>> for UnicodeSet<'data> {
     type Error = UnicodeSetError;
 
     fn try_from(range: &RangeToInclusive<char>) -> Result<Self, Self::Error> {
@@ -70,32 +76,52 @@ impl TryFrom<&RangeToInclusive<char>> for UnicodeSet {
     }
 }
 
+impl FromIterator<RangeInclusive<u32>> for UnicodeSet<'_> {
+    fn from_iter<I: IntoIterator<Item = RangeInclusive<u32>>>(iter: I) -> Self {
+        let mut builder = UnicodeSetBuilder::new();
+        for range in iter {
+            builder.add_range_u32(&range);
+        }
+        builder.build()
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use super::UnicodeSetError;
+    use super::*;
     use crate::UnicodeSet;
     use core::{char, convert::TryFrom};
 
     #[test]
     fn test_try_from_range() {
-        let check: Vec<char> = UnicodeSet::try_from(&('A'..'B')).unwrap().iter_chars().collect();
+        let check: Vec<char> = UnicodeSet::try_from(&('A'..'B'))
+            .unwrap()
+            .iter_chars()
+            .collect();
         assert_eq!(vec!['A'], check);
     }
+
     #[test]
     fn test_try_from_range_error() {
         let check = UnicodeSet::try_from(&('A'..'A'));
         assert!(matches!(check, Err(UnicodeSetError::InvalidRange(65, 65))));
     }
+
     #[test]
     fn test_try_from_range_inclusive() {
-        let check: Vec<char> = UnicodeSet::try_from(&('A'..='A')).unwrap().iter_chars().collect();
+        let check: Vec<char> = UnicodeSet::try_from(&('A'..='A'))
+            .unwrap()
+            .iter_chars()
+            .collect();
         assert_eq!(vec!['A'], check);
     }
+
     #[test]
     fn test_try_from_range_inclusive_err() {
         let check = UnicodeSet::try_from(&('B'..'A'));
         assert!(matches!(check, Err(UnicodeSetError::InvalidRange(66, 65))));
     }
+
     #[test]
     fn test_try_from_range_from() {
         let uset = UnicodeSet::try_from(&('A'..)).unwrap();
@@ -103,6 +129,7 @@ mod tests {
         let expected: usize = (char::MAX as usize) + 1 - 65;
         assert_eq!(expected, check);
     }
+
     #[test]
     fn test_try_from_range_to() {
         let uset = UnicodeSet::try_from(&(..'A')).unwrap();
@@ -110,11 +137,13 @@ mod tests {
         let expected: usize = 65;
         assert_eq!(expected, check);
     }
+
     #[test]
     fn test_try_from_range_to_err() {
         let check = UnicodeSet::try_from(&(..(0x0 as char)));
         assert!(matches!(check, Err(UnicodeSetError::InvalidRange(0, 0))));
     }
+
     #[test]
     fn test_try_from_range_to_inclusive() {
         let uset = UnicodeSet::try_from(&(..='A')).unwrap();
@@ -122,6 +151,7 @@ mod tests {
         let expected: usize = 66;
         assert_eq!(expected, check);
     }
+
     #[test]
     fn test_try_from_range_full() {
         let uset = UnicodeSet::try_from(&(..)).unwrap();
@@ -129,4 +159,18 @@ mod tests {
         let expected: usize = (char::MAX as usize) + 1;
         assert_eq!(expected, check);
     }
+
+    #[test]
+    fn test_from_range_iterator() {
+        let ranges: Vec<RangeInclusive<u32>> = vec![
+            RangeInclusive::new(0, 0x3FFF),
+            RangeInclusive::new(0x4000, 0x7FFF),
+            RangeInclusive::new(0x8000, 0xBFFF),
+            RangeInclusive::new(0xC000, 0xFFFF),
+        ];
+        let ranges_iter = ranges.into_iter();
+        let expected = UnicodeSet::from_inversion_list_slice(&[0x0, 0x1_0000]).unwrap();
+        let actual = UnicodeSet::from_iter(ranges_iter);
+        assert_eq!(expected, actual);
+    }
 }
