@@ -2,7 +2,7 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-//! `icu_uniset` is one of the [`ICU4X`] components.
+//! `icu_uniset` is a utility crate of the [`ICU4X`] project.
 //!
 //! This API provides necessary functionality for highly efficient querying of sets of Unicode characters.
 //!
@@ -22,7 +22,7 @@
 //! the [`UnicodeSetBuilder`], or from the TBA Properties API.
 //!
 //! ```
-//! use icu::uniset::{UnicodeSet, UnicodeSetBuilder};
+//! use icu_uniset::{UnicodeSet, UnicodeSetBuilder};
 //!
 //! let mut builder = UnicodeSetBuilder::new();
 //! builder.add_range(&('A'..'Z'));
@@ -36,7 +36,7 @@
 //! Currently, you can check if a character/range of characters exists in the [`UnicodeSet`], or iterate through the characters.
 //!
 //! ```
-//! use icu::uniset::{UnicodeSet, UnicodeSetBuilder};
+//! use icu_uniset::{UnicodeSet, UnicodeSetBuilder};
 //!
 //! let mut builder = UnicodeSetBuilder::new();
 //! builder.add_range(&('A'..'Z'));
@@ -49,49 +49,45 @@
 //!
 //! [`ICU4X`]: ../icu/index.html
 
+// https://github.com/unicode-org/icu4x/blob/main/docs/process/boilerplate.md#library-annotations
 #![cfg_attr(not(any(test, feature = "std")), no_std)]
+#![cfg_attr(
+    not(test),
+    deny(
+        clippy::indexing_slicing,
+        clippy::unwrap_used,
+        clippy::expect_used,
+        clippy::panic
+    )
+)]
+#![warn(missing_docs)]
 
 extern crate alloc;
 
 #[macro_use]
 mod builder;
 mod conversions;
-pub mod enum_props;
-pub mod props;
-pub mod provider;
 mod uniset;
 mod utils;
 
 use alloc::vec::Vec;
+
 pub use builder::UnicodeSetBuilder;
 pub use conversions::*;
 use displaydoc::Display;
-use icu_provider::DataError;
 pub use uniset::UnicodeSet;
 pub use utils::*;
 
 /// Custom Errors for [`UnicodeSet`].
 #[derive(Display, Debug)]
 pub enum UnicodeSetError {
+    /// A UnicodeSet was constructed with an invalid inversion list
     #[displaydoc("Invalid set: {0:?}")]
     InvalidSet(Vec<u32>),
+    /// A UnicodeSet was constructed containing an invalid range
     #[displaydoc("Invalid range: {0}..{1}")]
     InvalidRange(u32, u32),
-    #[displaydoc("{0}")]
-    PropDataLoad(DataError),
 }
 
 #[cfg(feature = "std")]
 impl std::error::Error for UnicodeSetError {}
-
-impl From<DataError> for UnicodeSetError {
-    fn from(e: DataError) -> Self {
-        UnicodeSetError::PropDataLoad(e)
-    }
-}
-
-#[derive(PartialEq)]
-pub enum UnicodeSetSpanCondition {
-    Contained,
-    NotContained,
-}
